@@ -10,82 +10,76 @@ from slimta.envelope import Envelope
 
 class TestPipeRelay(MoxTestBase, unittest.TestCase):
 
-    def test_exec_process(self):
+    def _mock_popen(self, rcpt, returncode, stdout):
         pmock = self.mox.CreateMock(subprocess.Popen)
-        self.mox.StubOutWithMock(subprocess, 'Popen')
-        env = Envelope('sender@example.com', ['rcpt@example.com'])
-        env.parse(b'From: sender@example.com\r\n\r\ntest test\r\n')
-        subprocess.Popen(['relaytest', '-f', 'sender@example.com'],
+        subprocess.Popen(['relaytest', '-f', 'sender@example.com', rcpt],
                          stdin=subprocess.PIPE,
                          stdout=subprocess.PIPE,
                          stderr=subprocess.PIPE).AndReturn(pmock)
-        pmock.communicate(b'From: sender@example.com\r\n\r\ntest test\r\n').AndReturn(('testout', 'testerr'))
+        pmock.communicate(b'From: sender@example.com\r\n\r\ntest test\r\n').AndReturn((stdout, ''))
         pmock.pid = -1
-        pmock.returncode = 0
-        self.mox.ReplayAll()
-        m = PipeRelay(['relaytest', '-f', '{sender}'])
-        status, stdout, stderr = m._exec_process(env)
-        self.assertEqual(0, status)
-        self.assertEqual('testout', stdout)
-        self.assertEqual('testerr', stderr)
+        pmock.returncode = returncode
+        return pmock
 
-    def test_exec_process_error(self):
-        pmock = self.mox.CreateMock(subprocess.Popen)
+    def test_attempt(self):
+        self.mox.StubOutWithMock(subprocess, 'Popen')
+        env = Envelope('sender@example.com', ['rcpt1@example.com', 'rcpt2@example.com', 'rcpt3@example.com', 'rcpt4@example.com'])
+        env.parse(b'From: sender@example.com\r\n\r\ntest test\r\n')
+        self._mock_popen('rcpt1@example.com', 0, '')
+        self._mock_popen('rcpt2@example.com', 1337, 'transient')
+        self._mock_popen('rcpt3@example.com', 1337, '5.0.0 permanent')
+        subprocess.Popen(['relaytest', '-f', 'sender@example.com', 'rcpt4@example.com'],
+                         stdin=subprocess.PIPE,
+                         stdout=subprocess.PIPE,
+                         stderr=subprocess.PIPE).AndRaise(Timeout)
+        self.mox.ReplayAll()
+        m = PipeRelay(['relaytest', '-f', '{sender}', '{recipient}'])
+        results = m.attempt(env, 0)
+        self.assertEqual(4, len(results))
+        self.assertEqual(None, results['rcpt1@example.com'])
+        self.assertIsInstance(results['rcpt2@example.com'], TransientRelayError)
+        self.assertEqual('transient', str(results['rcpt2@example.com']))
+        self.assertEqual('450', results['rcpt2@example.com'].reply.code)
+        self.assertIsInstance(results['rcpt3@example.com'], PermanentRelayError)
+        self.assertEqual('5.0.0 permanent', str(results['rcpt3@example.com']))
+        self.assertEqual('550', results['rcpt3@example.com'].reply.code)
+        self.assertIsInstance(results['rcpt4@example.com'], TransientRelayError)
+        self.assertEqual('Delivery timed out', str(results['rcpt4@example.com']))
+        self.assertEqual('450', results['rcpt4@example.com'].reply.code)
+
+
+class TestMaildropRelay(MoxTestBase, unittest.TestCase):
+
+    def test_attempt(self):
         self.mox.StubOutWithMock(subprocess, 'Popen')
         env = Envelope('sender@example.com', ['rcpt@example.com'])
         env.parse(b'From: sender@example.com\r\n\r\ntest test\r\n')
-        subprocess.Popen(['relaytest', '-f', 'sender@example.com'],
+        pmock = self.mox.CreateMock(subprocess.Popen)
+        subprocess.Popen(['maildrop', '-f', 'sender@example.com'],
                          stdin=subprocess.PIPE,
                          stdout=subprocess.PIPE,
                          stderr=subprocess.PIPE).AndReturn(pmock)
         pmock.communicate(b'From: sender@example.com\r\n\r\ntest test\r\n').AndReturn(('', ''))
         pmock.pid = -1
-        pmock.returncode = 1337
+        pmock.returncode = 0
         self.mox.ReplayAll()
-        m = PipeRelay(['relaytest', '-f', '{sender}'])
-        status, stdout, stderr = m._exec_process(env)
-        self.assertEqual(1337, status)
-        self.assertEqual('', stdout)
-        self.assertEqual('', stderr)
-
-    def test_attempt(self):
-        env = Envelope()
-        m = PipeRelay(['relaytest'])
-        self.mox.StubOutWithMock(m, '_exec_process')
-        m._exec_process(env).AndReturn((0, '', ''))
-        self.mox.ReplayAll()
-        m.attempt(env, 0)
-
-    def test_attempt_transientfail(self):
-        env = Envelope()
-        m = PipeRelay(['relaytest'])
-        self.mox.StubOutWithMock(m, '_exec_process')
-        m._exec_process(env).AndReturn((1337, 'transient failure', ''))
-        self.mox.ReplayAll()
-        with self.assertRaises(TransientRelayError):
-            m.attempt(env, 0)
+        m = MaildropRelay()
+        result = m.attempt(env, 0)
+        self.assertEqual(None, result)
 
     def test_attempt_timeout(self):
-        env = Envelope()
-        m = PipeRelay(['relaytest'])
-        self.mox.StubOutWithMock(m, '_exec_process')
-        m._exec_process(env).AndRaise(Timeout)
+        self.mox.StubOutWithMock(subprocess, 'Popen')
+        env = Envelope('sender@example.com', ['rcpt@example.com'])
+        env.parse(b'From: sender@example.com\r\n\r\ntest test\r\n')
+        subprocess.Popen(['maildrop', '-f', 'sender@example.com'],
+                         stdin=subprocess.PIPE,
+                         stdout=subprocess.PIPE,
+                         stderr=subprocess.PIPE).AndRaise(Timeout)
         self.mox.ReplayAll()
+        m = MaildropRelay()
         with self.assertRaises(TransientRelayError):
             m.attempt(env, 0)
 
-    def test_attempt_permanentfail(self):
-        env = Envelope()
-        m = PipeRelay(['relaytest'])
-        self.mox.StubOutWithMock(m, '_exec_process')
-        m._exec_process(env).AndReturn((13, '5.0.0 permanent failure', ''))
-        self.mox.ReplayAll()
-        with self.assertRaises(PermanentRelayError):
-            m.attempt(env, 0)
-
-
-class TestMaildropRelay(MoxTestBase, unittest.TestCase):
-
     def test_extra_args(self):
         m = MaildropRelay(extra_args=['-t', 'test'])
         self.assertEquals(['-t', 'test'], m.args[-2:])
